@@ -5,9 +5,10 @@ use MojoX::DirectoryListing::Icons;
 use strict;
 use warnings FATAL => 'all';
 use base 'Exporter';
+use Cwd;
 
 our @EXPORT = ('serve_directory_listing');
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 # FIXME: see  @{app->static->paths} for list of public directories
 our $public_dir = "public";
@@ -18,14 +19,23 @@ sub set_public_app_dir {
     $public_dir =~ s{/+$}{};
 }
 
+my %realpaths;
+
 sub serve_directory_listing {
+    %realpaths = ();
     my $route = shift;
     my $local;
     if (@_ % 2 == 1) {
 	$local = shift;
     }
+    _serve_directory_listing($route, $local, 'caller', caller, @_);
+}
+
+sub _serve_directory_listing {
+    my $route = shift;
+    my $local = shift;
     my %options = @_;
-    my $caller = $options{caller} // caller;
+    my $caller = $options{caller};
 
     if ($route !~ m{^/}) {
 	$caller->app->log->error(
@@ -53,10 +63,12 @@ sub serve_directory_listing {
 	my $route1 = $route eq '/' ? '' : $route;
 	foreach my $subdir (@subdirs) {
 	    if ($local) {
-		serve_directory_listing( "$route1/$subdir",
+		my $real = Cwd::realpath("$local/$subdir");
+		next if $realpaths{$real}++;
+		_serve_directory_listing( "$route1/$subdir",
 					 "$local/$subdir", %options );
 	    } else {
-		serve_directory_listing( "$route1/$subdir", %options );
+		_serve_directory_listing( "$route1/$subdir", undef, %options );
 	    }
 	}
     }
@@ -224,7 +236,7 @@ sub _render_directory {
 	} else {
 
 	    $output .= qq[  <th>
-      <a class="directory-listing-link" href="?C=$col_code;O=$order_code">$text</a> $sortind
+      <a class="directory-listing-link" href="$virtual_dir?C=$col_code;O=$order_code">$text</a> $sortind
   </th>
 ];
 	}
@@ -389,7 +401,7 @@ MojoX::DirectoryListing - show Apache-style directory listings in your Mojolicio
 
 =head1 VERSION
 
-0.05
+0.06
 
 =head1 SYNOPSIS
 
@@ -619,22 +631,6 @@ The public app dir is used to map the route to an actual
 path when you don't supply a C<$path> argument to
 L<"serve_directory_listing">.
 
-=head1 TODO
-
-There are a lot of ways this module could be more powerful and more
-useful:
-
-=over 4
-
-=item * user-configurable page header
-
-=item * user-configurable page footer
-
-=item * flesh out and document classes that can be
-styled with CSS
-
-=back
-
 =head1 AUTHOR
 
 Marty O'Brien, C<< <mob at cpan.org> >>
